/****************************************************************************************
*	AETEGizmo.c
*
*	Copyright  1998 Red Shed Software. All rights reserved.
*	by Jonathan 'Wolf' Rentzsch (jon@redshed.net)
*
*	Commenter	Date				Comment
*	---------	-----------------	-----------------------------------------------------
*	wolf		Wed, Jun 17, 1998	Created.
*	
****************************************************************************************/

#include	"AETEGizmo.h"
#include	"AEPrint.h"
#include	"BuildUPP.h"
#include	"wolf.h"
#include	"AppleEventUtilities.h"
#include	"MacExceptions.h"
#include	"ASRegistry.h"

			pascal	OSErr
HandleEveryEvent(	TAEvent	*event,
					TAEvent	*reply,
					long	refCon );
DeclareAEEventHandlerProc( gHandleEveryEvent, (AEEventHandlerUPP) HandleEveryEvent );

bool	gDone = false;
Handle	gAETE = nil;
NMRec	gNM;

/****************************************************************************************
	Commenter	Date				Comment
	---------	-----------------	-----------------------------------------------------
	wolf		Wed, Jun 17, 1998	Created.
	
	************************************************************************************/

	void
main()
{
	OSErr	err = noErr;
	
	MaxApplZone();
	InitGraf((Ptr) &qd.thePort );
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( 0L );
	InitCursor();
	FlushEvents( everyEvent, 0 );
	
	if( !err ) {
		Handle	menubar = GetNewMBar( kMenubar );
		if( menubar == nil )
			err = resNotFound;
		if( !err ) {
			SetMenuBar( menubar );
			DisposeHandle( menubar );
			AppendResMenu( GetMenuHandle( kAppleMenu ), 'DRVR' );
			DrawMenuBar();
		}
	}
	
	if( !err )
		err = AEInstallEventHandler( typeWildCard, typeWildCard, gHandleEveryEvent, 0, false );
	if( !err ) {
		EventRecord	event;
		while( !gDone ) {
			(void) WaitNextEvent( everyEvent, &event, GetCaretTime(), nil );
			switch( event.what ) {
				case mouseDown:
					WindowPtr	whichWindow;
					switch( FindWindow( event.where, &whichWindow ) ) {
						case inMenuBar:
							long	menuCommand = MenuSelect( event.where );
							short	menu = HiWrd( menuCommand );
							short	item = LoWrd( menuCommand );
							switch( menu ) {
								case kAppleMenu:
									switch( item ) {
										case kAboutItem:
											gNM.qType = nmType;
											gNM.nmMark = 0;
											gNM.nmIcon = nil;
											gNM.nmSound = nil;
											gNM.nmStr = "\pAETEGizmo 1.0 by Jonathan Wolf Rentzsch (jon@redshed.net)\r\rThanks to Jens Alfke for AEGizmos!";
											gNM.nmResp = nil;
											NMInstall( &gNM );
											break;
										default:
											Str255	daName;
											GetMenuItemText( GetMenuHandle( kAppleMenu ), item, daName );
											(void) OpenDeskAcc( daName );
											break;
									}
									break;
								case kFileMenu:
									switch( item ) {
										case kQuitItem:
											gDone = true;
											break;
										default:
											break;
									}
									break;
								case kEditMenu:
									break;
								default:
									break;
							}
							HiliteMenu( 0 );
							break;
						case inSysWindow:
							SystemClick( &event, whichWindow );
							break;
						case inContent:
							break;
						case inDrag:
							break;
						case inGrow:
							break;
						case inGoAway:
							break;
						case inZoomIn:
						case inZoomOut:
							break;
					}
					break;
				case keyDown:
					if( event.modifiers & cmdKey && (event.message & charCodeMask) == 'q' )
						gDone = true;
					break;
				case autoKey:
					break;
				case diskEvt:
					if( HiWrd( event.message ) ) {
						DILoad();
						SetPt( &event.where, 120, 120 );
						(void) DIBadMount( event.where, event.message );
						DIUnload();
					}
					break;
				case kHighLevelEvent:
					AEProcessAppleEvent( (EventRecord*)&event );
					break;
				case updateEvt:
					break;
				case activateEvt:
					break;
				case osEvt:
					break;
				default:
					break;
			}
		}
	}
}

/****************************************************************************************
	Commenter	Date				Comment
	---------	-----------------	-----------------------------------------------------
	wolf		Wed, Jun 17, 1998	Created.
	
	************************************************************************************/

			pascal	OSErr
HandleEveryEvent(	TAEvent	*event,
					TAEvent	*reply,
					long	refCon )
{
#pragma	unused( refCon )
	try {
		DescType	eventClass = event->GetTypeAttribute( keyEventClassAttr );
		DescType	eventID = event->GetTypeAttribute( keyEventIDAttr );
		
		if( eventClass == kCoreEventClass && eventID == kAEOpenDocuments ) {
			TDescriptor	docList = event->GetDescriptor( keyDirectObject, typeAEList );
			if( docList.CountItems() != 1 )
				throw( errAECorruptData );
			TDescriptor	doc = docList.GetNthDescriptor( 1, typeFSS );
			FSSpec	docSpec;
			doc.GetFSS( docSpec );
			SetResLoad( false );
			short	docSpecResID = FSpOpenResFile( &docSpec, fsRdPerm );
			SetResLoad( true );
			if( docSpecResID > 0 ) {
				if( gAETE )
					DisposeHandle( gAETE );
				gAETE = Get1Resource( 'aete', 0 );
				ThrowIfResFail( gAETE );
				DetachResource( gAETE );
				HLockHi( gAETE );
				CloseResFile( docSpecResID );
			}
			docList.Dispose();
			doc.Dispose();
		} else if( eventClass == kCoreEventClass && eventID == kAEQuitApplication ) {
			gDone = true;
		} else if( eventClass == kASAppleScriptSuite && eventID == kGetAETE ) {
			if( gAETE )
				reply->PutParameterPtr( keyDirectObject, typeAETE, *gAETE, GetHandleSize( gAETE ));
			else
				return( errAEEventNotHandled );
		} else {
			if( !reply->IsNullDescriptor() ) {
				long	l;
				ThrowIfErr( AEPrintSize( (AEDesc*) event, &l ) );
				Handle	h = NewHandle( l );
				ThrowIfMemFail( h );
				HLockHi( h );
				ThrowIfErr( AEPrint( (AEDesc*) event, *h, l ) );
				reply->PutParameterPtr( keyDirectObject, typeChar, *h, l - 1 );
				DisposeHandle( h );
			}
		}
	} catch( OSErr err ) {
		return( err );
	}
	
	return( noErr );
}