//============================================================================
// COutlineTable.cp		1997 Metrowerks Inc. All rights reserved
// Original author: John C. Daub
//============================================================================
//	Subclass and concrete instantiation of LOutlineTable

#include "COutlineTable.h"

#include <LTableMultiGeometry.h>
#include <LOutlineKeySelector.h>
#include <LOutlineRowSelector.h>
#include <LOutlineItem.h>
#include <UAttachments.h>

#include <Gestalt.h>

#include "CDirectoryItem.h"
#include "CFileItem.h"

#include "OutlineTablePrefix.h"

extern Int16 gVolIndex;	// defined in COutlineTableApp.cp

#define RezType_Gestalt	'gest'

typedef struct _GestaltInfo
{
	OSType		selector;
	Uint32		bits;
	long		nameId;
} GestaltInfo;

typedef struct _GestaltInfoResource
{
	long		count;
	GestaltInfo	items[];
} GestaltInfoResource, *GestaltInfoResPtr, **GestaltInfoResHndl;

// Constructor

COutlineTable::COutlineTable(
	LStream *inStream )
		: LOutlineTable( inStream )
{
	// set the table geometry
	
	SetTableGeometry(new LTableMultiGeometry(this, kColWidth, 20));
	
	// set the table selector
	
	SetTableSelector(new LOutlineRowSelector( this ) );
	
	// and note that we don't set the table storage....
	
	// most of the table classes not only maintain the graphical
	// representation of your data but also the data itself. But
	// LOutlineTable doesn't really do this...it mostly handles
	// graphical representation... you need to handle your data
	// maintenance elsewhere by yourself.
	
	// insert a couple columns (name and size)
	
	InsertCols( 1, 0, nil, nil, false );

	// Set up keyboard selection and scrolling.

	AddAttachment(new LOutlineKeySelector(this, msg_AnyMessage));
	AddAttachment(new LKeyScrollAttachment(this));

	// Try to become default commander in the window.

	if (mSuperCommander != nil) {
		mSuperCommander->SetLatentSub(this);
	}

}

// dtor

COutlineTable::~COutlineTable()
{
	// nothing
}


// FinishCreateSelf

void
COutlineTable::FinishCreateSelf()
{
	// create the initial detail of the table

	// get the volume the user specified in the dialog
#if 000000	
	HParamBlockRec	pb;
	Str31			volName;
	pb.volumeParam.ioNamePtr	= volName;
	pb.volumeParam.ioVolIndex	= gVolIndex;
	
	::PBHGetVInfoSync( &pb );
#endif
	// insert the root level of the disk into the table
		
	LOutlineItem *theItem = nil;
	LOutlineItem *lastItem = nil;
	Int16 index = 1;

	// loop through every item at this level
#if 000000	
	while ( true ) {
		
		// find the file at the given index
		
		CInfoPBRec fileInfo;
		Str31 fileName;
		
		fileInfo.hFileInfo.ioCompletion = nil;
		fileInfo.hFileInfo.ioNamePtr = fileName;
		fileInfo.hFileInfo.ioVRefNum = pb.volumeParam.ioVRefNum;
		fileInfo.hFileInfo.ioFDirIndex = index++;
		fileInfo.hFileInfo.ioDirID = 0;

		OSErr err = ::PBGetCatInfoSync( &fileInfo );
		if ( err != noErr )
			break;

		// we ignore invis stuff
		if ( fileInfo.hFileInfo.ioFlFndrInfo.fdFlags & kIsInvisible )
			continue;

		// make an FSSpec for the file
		
		FSSpec theSpec;
		
		ThrowIfOSErr_(::FSMakeFSSpec( pb.volumeParam.ioVRefNum,
						0,
						fileName,
						&theSpec ));
		
		// make an outline item based upon a directory or file.
		// LOutlineTable uses LOutlineItems to represent the rows
		// within the table. This allows a great deal of flexibilty in
		// terms of what is displayed in each row (e.g. a directory or
		// a file or whatever).
#endif		

		GestaltInfoResHndl		gestInfoRes;
		long					count = 0;
		long					i;

		gestInfoRes = (GestaltInfoResHndl)GetResource(RezType_Gestalt, 0);

		if (gestInfoRes && (ResError() == noErr))
		{
			count = (**gestInfoRes).count;
		}

		for (i = 0; i < count; i++)
		{
			// it's a gestalt selector
			
			theItem =
				new CDirectoryItem
				(
					(**gestInfoRes).items[i].nameId,
					(**gestInfoRes).items[i].selector,
					(**gestInfoRes).items[i].bits
				);

			ThrowIfNil_(theItem);

			// and insert it at the end of the table

			InsertItem( theItem, nil, lastItem );
			lastItem = theItem;
		}

#if 000000
typedef struct _GestaltInfo
{
	OSType		selector;
	Uint32		bits;
	long		nameId;
} GestaltInfo;

typedef struct _GestaltInfoResource
{
	long		count;
	GestaltInfo	items[];
	}
#endif
}
