/*
**	Y2K was a hack written for MacHack 1988 by Simon Fraser and John McMullen,
**	extending the original idea for Y2K from Jon Kalb.
**
**	Y2K was implemented by starting with the Apple DTS sample code
**	The original copyright blurb appears below.
*/

/*
**	
**	Apple Macintosh Developer Technical Support
**
**	Sample code demonstrating how to patch PowerOff key
**
**	by Brian Bechtel, Apple Developer Technical Support
**
**	File:		PatchPowerOff.c
**
**	Copyright  1995 Apple Computer, Inc.
**	All rights reserved.
**
**	You may incorporate this sample code into your applications without
**	restriction, though the sample code has been provided "AS IS" and the
**	responsibility for its operation is 100% yours.  However, what you are
**	not permitted to do is to redistribute the source as "DTS Sample Code"
**	after having made changes. If you're going to re-distribute the source,
**	we require that you make it clear in the source that the code was
**	descended from Apple Sample Code, but that you've made changes.
*/
#include <Gestalt.h>
#include <Resources.h>
#include <Errors.h>
#include <OSUtils.h>
#include <Traps.h>
#include <SetupA4.h>
#include <A4Stuff.h>

#include "updatepatch.h"

/* for debugging */
#define assert(i) if ((i) DebugStr("\p;printf \"assertion failed file %s line %d\" __FILE__, __LINE__")

#define kINITid			128	/* matches the resID in the 68K Project preferences */
#define kICONid			128

//#define USE_DEBUGGER_CALLS

pascal void ShowIcon7(short iconId, Boolean advance);

// local function declarations
void InitBeginUpdatePatch(void);
void ClearBeginUpdatePatch(void);
#if TESTING_PATCH
pascal void asm BeginUpdatePatch(WindowPtr wind);
#endif // TESTING_PATCH

void main(void);

static long		gOriginalTrapPtr = 0;
static Boolean	gBeginUpdatePatchIsInstalled = false;

/*
 * InstallBeginUpdatePatch
 * This routine patches the BeginUpdate() trap to draw different stuff - maybe.
 */
void InitBeginUpdatePatch(void)
{
	if (gBeginUpdatePatchIsInstalled == false)
	{
#if TESTING_PATCH
		//gOriginalTrapPtr = (long) NGetTrapAddress( _BeginUpDate, ToolTrap);
		//NSetTrapAddress( (UniversalProcPtr)(BeginUpdatePatch), _BeginUpDate, ToolTrap);
#else
		InstallPatch();
#endif // TESTING_PATCH

		gBeginUpdatePatchIsInstalled = true;
	}
}

/*
 * ClearBeginUpdatePatch
 * This routine clears out the patch we installed in InstallBeginUpdatePatch.
 * Note: Never call this routine from within an extension; you don't know
 * what other patches may have been put on Alert() after you installed
 * your patch, and bad things would happen if you eliminate yourself from
 * the chain of patches...
 */
void ClearBeginUpdatePatch(void)
{
	if (gBeginUpdatePatchIsInstalled == true)
	{
		NSetTrapAddress( (UniversalProcPtr)(gOriginalTrapPtr), _Alert, ToolTrap);
		gBeginUpdatePatchIsInstalled = false;
	}
}

#if TESTING_PATCH
/*
 * BeginUpdatePatch
 */
pascal void asm BeginUpdatePatch(WindowPtr wind)
{
// Check for proper alert.  If it's the trap we want to avoid,
// return to caller without actually executing the Alert trap.
// Set up the stack so it looks as if the user hit okay.

/*
PatchBeginUpdate:
	clr.l	-(SP)
	clr.l	-(SP)
	clr.l	-(SP)
	jsr		_SysBeep
	jsr		_SysBeep
	jsr		_SysBeep
*/

exitPatchBeginUpdate:
	// call the original trap.
//First, set up a4 to access the old trap address
	jsr		SetUpA4		// puts old a4 into register d0

	lea		gOriginalTrapPtr, a0

	exg		d0, a4		// restore old value of a4
	
	movea.l	(a0), a0
	jmp		(a0)	
}
#endif // TESTING_PATCH

/* main */

void main(void)
{
	long	oldA4;
	Handle	h;
	OSErr	err = noErr;
	
	#ifdef USE_DEBUGGER_CALLS
		Debugger();
	#endif

	/* set up our A4 context for _this file_ */
	oldA4 = SetCurrentA4();
	RememberA4();
		
	/* detach ourselves */
	h = Get1Resource('INIT', kINITid);
	if (h) DetachResource(h);
		
	InitBeginUpdatePatch();

	ShowIcon7(kICONid, true);
	
	/* restore the a4 world */
	SetA4(oldA4);
}
