

#include "updatepatch.h"


#include <SetupA4.h>
#include <A4Stuff.h>

#define kBrokenImageIconID			319
#define kUnfoundImageIconID			318

#define kBrokenPluginIconID			326
#define kUnfoundPluginIconID		325

#define Yr2KResID					128
#define Yr2KResourceType			'Yr2K'

#define kMaxWindArrayLength			128
#define kNumPatchedWindows			10

typedef pascal void (*BeginUpdateCall)(WindowPtr wind);

typedef struct {
	long		windowPatchFreq;
	long		unused1;
	long		unused2;
	Str255		appName;		// may be < 255 chars of data
} TResourceData, *TResourceDataPtr, **TResourceDataHandle;

typedef struct {
	CIconHandle		theIcon;
	
	TResourceData	resourceData;

	long			repeatFrequency;
	
	long			nextUnpatchedWindowIndex;
	long			nextPatchedWindowIndex;
	
	WindowPtr		unpatchedWindowList[kMaxWindArrayLength];
	WindowPtr		patchedWindows[kNumPatchedWindows];
	
} TUpdatePatchData;


TUpdatePatchData	gUpdatePatchData = {0};
UniversalProcPtr	gBeginUpdateCallThru = 0L;

void InstallPatch(void)
{
	TResourceDataHandle		resHand = nil;
	Zone* savedZone;
	
	resHand = (TResourceDataHandle)GetResource(Yr2KResourceType, Yr2KResID);
	if (resHand == nil)
		return;		// don't do any patches
	
	gUpdatePatchData.resourceData = **resHand;
	
	savedZone = GetZone();
	SetZone(SystemZone());
	gUpdatePatchData.theIcon = GetCIcon(kBrokenImageIconID);
	
	gUpdatePatchData.repeatFrequency = gUpdatePatchData.resourceData.windowPatchFreq;
	
	if (gUpdatePatchData.repeatFrequency > kNumPatchedWindows)
		gUpdatePatchData.repeatFrequency = kNumPatchedWindows;
	
	gUpdatePatchData.nextPatchedWindowIndex = 0;
	gUpdatePatchData.nextUnpatchedWindowIndex = 0;
	
	gBeginUpdateCallThru = NGetTrapAddress(_BeginUpDate, ToolTrap);
	NSetTrapAddress((UniversalProcPtr)BeginUpdatePatch, _BeginUpDate, ToolTrap);
	
	SetZone(savedZone);
}



static Boolean IsWindowPatched(WindowPtr wind)
{
	short		i;
	
	for (i = 0; i < kNumPatchedWindows; i ++)
	{
		if (wind == gUpdatePatchData.patchedWindows[i])
			return true;
	}
	
	return false;
}

static Boolean IsWindowInList(WindowPtr wind)
{
	short		i;
	
	for (i = 0; i < gUpdatePatchData.repeatFrequency; i ++)
	{
		if (wind == gUpdatePatchData.unpatchedWindowList[i])
			return true;
	}
	
	return false;
}

static void AddWindowToPatchedList(WindowPtr wind)
{
	gUpdatePatchData.patchedWindows[gUpdatePatchData.nextPatchedWindowIndex] = wind;
	gUpdatePatchData.nextPatchedWindowIndex ++;
	
	if (gUpdatePatchData.nextPatchedWindowIndex == kNumPatchedWindows)
		gUpdatePatchData.nextPatchedWindowIndex = 0;
	
}

static Boolean AddWindowToUnpatchedList(WindowPtr wind)
{
	if (gUpdatePatchData.nextUnpatchedWindowIndex == gUpdatePatchData.repeatFrequency)
	{
		gUpdatePatchData.nextUnpatchedWindowIndex = 0;
		return false;
	}
		
	gUpdatePatchData.unpatchedWindowList[gUpdatePatchData.nextUnpatchedWindowIndex] = wind;
	gUpdatePatchData.nextUnpatchedWindowIndex ++;
	
	return true;
}


#define kDocWindowFlagMask		(1 << 2)

// is this a window we want to mess with?
static Boolean IsPatchableWindow(WindowPtr wind)
{
	//WindowPeek		windPeek = WindowPeek(wind);
	AuxWinHandle	auxWin;
	
	if (GetAuxWin(wind, &auxWin))
		return ((**auxWin).awFlags & kDocWindowFlagMask) != 0;
	else
		return false;
}


static void DrawServerErrorGraphic(WindowPtr wind)
{
	Rect			imageRect;
	Rect			iconRect = {0, 0, 32, 32};
	RGBColor		oldBackColor, oldForeColor;
	RGBColor		netColor = {0xCCCC, 0xCCCC, 0xCCCC};
	RGBColor		rgbBlackColor = {0, 0, 0};

	imageRect = wind->portRect;
	
	GetBackColor(&oldBackColor);
	GetForeColor(&oldForeColor);
	
	RGBBackColor(&netColor);
	EraseRect(&wind->portRect);
	
	RGBForeColor(&rgbBlackColor);

	TextFont(kFontIDTimes);
	MoveTo(10, 30);
	TextSize(24);
	DrawString("\pObject not found");

	RGBForeColor(&oldForeColor);
	RGBBackColor(&oldBackColor);
}



static void DrawMissingImageGraphic(WindowPtr wind)
{
	Rect			imageRect;
	Rect			iconRect = {0, 0, 32, 32};
	RGBColor		oldBackColor, oldForeColor;
	RGBColor		netColor = {0xCCCC, 0xCCCC, 0xCCCC};
	RGBColor		rgbBlackColor = {0, 0, 0};
	
	imageRect = wind->portRect;
	InsetRect(&imageRect, 3, 3);
	
	GetBackColor(&oldBackColor);
	GetForeColor(&oldForeColor);
	
	RGBBackColor(&netColor);
	EraseRect(&wind->portRect);
	
	RGBForeColor(&rgbBlackColor);
	FrameRect(&imageRect);
	
	OffsetRect(&iconRect, 7, 7);
	
	PlotCIcon(&iconRect, gUpdatePatchData.theIcon);

	RGBForeColor(&oldForeColor);
	RGBBackColor(&oldBackColor);
}

pascal void BeginUpdatePatch(WindowPtr wind)
{
	BeginUpdateCall oldPatchCode;
	
	EnterCodeResource();
	
	oldPatchCode = (BeginUpdateCall)gBeginUpdateCallThru;
	
	if (! IsPatchableWindow(wind) )
		goto done;
	
	if (IsWindowPatched(wind))
	{
		GrafPtr		oldPort;
		
		GetPort(&oldPort);
		SetPort(wind);
		
		DrawMissingImageGraphic(wind);
		//DrawServerErrorGraphic(wind);
		
		// Set the visRgn to empty so that the app does not draw
		SetEmptyRgn(wind->visRgn);
		
		SetPort(oldPort);
	}
	else				// is it a new window?
	{
		if (! IsWindowInList(wind))
		{
			if (! AddWindowToUnpatchedList(wind))
			{
				AddWindowToPatchedList(wind);
			}
		}
	}

done:
	ExitCodeResource(); // restore A4
	(*oldPatchCode)(wind);
}