#include "Maynard.h"
#include "ComponentHelp.h"

#include "TVMediaHandlerComponent.h"
#include "TVMediaHandlerVersion.h"

///=============================================================================

#if STAND_ALONE
	#define TVMediaDispatchSelector main
#else
	Component RegisterTVMediaHandlerComponent(void);
#endif //STAND_ALONE
pascal ComponentResult TVMediaDispatchSelector(
  ComponentParameters* paramsPtr, Handle thiss);

///=============================================================================

#define STATIC static
#define COMPONENT 

static BOOL fUpdate=FALSE;

//General media handler functions.

STATIC COMPONENT ComponentResult TVMediaOpen(
  TVMediaHandlerRecord* thiss, ComponentInstance self);

STATIC COMPONENT ComponentResult TVMediaClose(
  TVMediaHandlerRecord* thiss, ComponentInstance self);

STATIC COMPONENT ComponentResult TVMediaTarget(
  TVMediaHandlerRecord* thiss, ComponentInstance parentComponent);

STATIC COMPONENT ComponentResult TVMediaCanDo(
  TVMediaHandlerRecord* thiss, short functionNumber);

STATIC COMPONENT ComponentResult TVMediaVersion(
  TVMediaHandlerRecord* thiss);



STATIC COMPONENT ComponentResult TVMediaInitialize(
  TVMediaHandlerRecord* thiss, GetMovieCompleteParams* gmcPtr);

STATIC COMPONENT ComponentResult TVMediaSetActive(
  TVMediaHandlerRecord* thiss, Boolean fEnableMedia);


STATIC COMPONENT ComponentResult TVMediaIdle(
  TVMediaHandlerRecord* thiss, TimeValue timeNow_mediaAxis, 
  UINT32 inFlags, UINT32* outFlagsPtr, TimeRecord const* timeNowTR_movieAxis);

STATIC COMPONENT ComponentResult TVMediaPreroll(
  TVMediaHandlerRecord* thiss, TimeValue time_mediaAxis, Fixed fxRate);

STATIC COMPONENT ComponentResult TVMediaGetNextStepTime( 
  TVMediaHandlerRecord* thiss, short flags, 
  TimeValue timeIn_mediaAxis, TimeValue* timeOut_mediaAxisPtr, 
  Fixed fxRate);



STATIC COMPONENT ComponentResult TVMediaSetSoundOutputComponent(
  TVMediaHandlerRecord* thiss, Component outputComponent);

STATIC COMPONENT ComponentResult TVMediaSetVolume(
  TVMediaHandlerRecord* thiss, short volume);

STATIC COMPONENT ComponentResult TVMediaSetSoundBalance(
  TVMediaHandlerRecord* thiss, short balance);



STATIC COMPONENT ComponentResult TVMediaSetGWorld(
  TVMediaHandlerRecord* thiss, CGrafPtr, GDHandle);

STATIC COMPONENT ComponentResult TVMediaSetDimensions(
  TVMediaHandlerRecord* thiss, Fixed fxWidth, Fixed fxHeight);

STATIC COMPONENT ComponentResult TVMediaSetMatrix(
  TVMediaHandlerRecord* thiss, MatrixRecord* matrixRecordPtr);

STATIC COMPONENT ComponentResult TVMediaSetClip(
  TVMediaHandlerRecord* thiss, RgnHandle);

STATIC COMPONENT ComponentResult TVMediaSetGraphicsMode(
  TVMediaHandlerRecord* thiss, long mode, RGBColor const* opColorPtr);



STATIC COMPONENT ComponentResult TVMediaSetMovieTimeScale(
  TVMediaHandlerRecord* thiss, TimeScale movieTimeScale);

STATIC COMPONENT ComponentResult TVMediaSetMediaTimeScale(
  TVMediaHandlerRecord* thiss, TimeScale mediaTimeScale);

STATIC COMPONENT ComponentResult TVMediaTrackEdited(
  TVMediaHandlerRecord* thiss);

STATIC COMPONENT ComponentResult TVMediaTrackReferencesChanged(
  TVMediaHandlerRecord* thiss);

STATIC COMPONENT ComponentResult TVMediaGSetActiveSegment(
  TVMediaHandlerRecord* thiss, 
  TimeValue activeSegmentStart, TimeValue activeSegmentDuration);

STATIC COMPONENT ComponentResult TVMediaSetRate(
  TVMediaHandlerRecord* thiss, Fixed fxRate);


STATIC COMPONENT ComponentResult TVMediaSetHints(
  TVMediaHandlerRecord* thiss, long hints);

STATIC COMPONENT ComponentResult TVMediaInvalidateRegion(
  TVMediaHandlerRecord* thiss, RgnHandle rgnH);



STATIC COMPONENT ComponentResult TVMediaHasCharacteristic(
  TVMediaHandlerRecord* thiss, OSType characteristic, Boolean* fHasItPtr);

STATIC COMPONENT ComponentResult TVMediaGetName(
  TVMediaHandlerRecord* thiss, unsigned char* name, 
  long requestedLanguage, long* actualLanguagePtr);

STATIC COMPONENT ComponentResult TVMediaGetClock(
  TVMediaHandlerRecord* thiss, ComponentInstance* clockPtr);

STATIC COMPONENT ComponentResult TVMediaMakeMediaTimeTable(
  TVMediaHandlerRecord* thiss, long** offsetsH, long* offsetCountPtr, 
  TimeValue startTime, TimeValue endTime, TimeValue timeIncrement, 
  long *maxOffsetOut);

STATIC COMPONENT ComponentResult TVMediaGetSoundOutputComponent(
  TVMediaHandlerRecord* thiss, Component* outputComponentPtr);

///*****************************************************************************

#if GENERATINGPOWERPC
//Note that while this struct may appear to be a static variable, it 
// actually MUST be exported because it is the entry point used by the 
// component manager when it connects to this code fragment.
struct RoutineDescriptor TVMediaSelectorDispatcher
  =BUILD_ROUTINE_DESCRIPTOR(
	kPascalStackBased
	|RESULT_SIZE(kFourByteCode) 
	|STACK_ROUTINE_PARAMETER(1,kFourByteCode) 
	|STACK_ROUTINE_PARAMETER(2,kFourByteCode),
	TVMediaDispatchSelector);
#endif //GENERATINGPOWERPC

///-----------------------------------------------------------------------------

#if !STAND_ALONE
Component RegisterTVMediaHandlerComponent(void)
{
#if GENERATINGPOWERPC
	#define componentEntryPoint &TVMediaSelectorDispatcher
	#define registrationFlags   0x200|registerComponentGlobal
	//Note the 0x200 tells RegisterComponent() we are PPC native.
#else
	#define componentEntryPoint TVMediaDispatchSelector
	#define registrationFlags   registerComponentGlobal
#endif

	ComponentDescription cd;
	Component            registeredComponent;

	cd.componentType        =MediaHandlerType;
	cd.componentManufacturer=0;
	cd.componentFlags       =0;
	cd.componentFlagsMask   =0;

	cd.componentSubType='mhTV';
	registeredComponent=NULL;
	do{
		registeredComponent=FindNextComponent(registeredComponent, &cd);
		UnregisterComponent(registeredComponent);
	}while(registeredComponent);

///.............................................................................

	cd.componentType        =MediaHandlerType;
	cd.componentManufacturer='TEST';
	cd.componentFlags       =0;
	cd.componentFlagsMask   =0;

	cd.componentSubType='mhTV';
	registeredComponent=RegisterComponent(&cd, 
	  componentEntryPoint, registrationFlags, 0, 0, 0);
	SetDefaultComponent(registeredComponent, 
	  defaultComponentAnyFlagsAnyManufacturer);

	return registeredComponent;
}
#endif //STAND_ALONE
///-----------------------------------------------------------------------------

pascal ComponentResult TVMediaDispatchSelector(
  ComponentParameters* paramsPtr, Handle thiss)
{
#define cmpGlobalParm REINTERPRET_CAST(TVMediaHandlerRecord*, thiss)

	ComponentResult error;

	switch (paramsPtr->what){
	///.........................................................................
	//Component manager selectors.
	case kComponentOpenSelect:
		error=TVMediaOpen( 
		  CMP1(ComponentInstance) );
		break;
	case kComponentCloseSelect:
		error=TVMediaClose( 
		  CMP1(ComponentInstance) );
		break;
	case kComponentTargetSelect: 
		error=TVMediaTarget( 
		  CMP1(ComponentInstance) );
		break;
	case kComponentCanDoSelect: 
		error=TVMediaCanDo( 
		  CMP1(short) );
		break;
	case kComponentVersionSelect: 
		error=TVMediaVersion( 
		  CMP0() );
		break;

	///.........................................................................
	//Media handler selectors.
	case kMediaInitializeSelect: 
		error=TVMediaInitialize( 
		  CMP1(GetMovieCompleteParams*) );
		break;
	case kMediaSetActiveSelect: 
		error=TVMediaSetActive( 
		  CMP1(Boolean) );
		break;

	case kMediaIdleSelect: 
		error=TVMediaIdle( 
		  CMP4(TimeValue, UINT32, UINT32*, TimeRecord const*) );
		break;
	case kMediaPrerollSelect: 
		error=TVMediaPreroll( 
		  CMP2(TimeValue, Fixed) );
		break;
	case kMediaGetNextStepTimeSelect:
		error=TVMediaGetNextStepTime( 
		  CMP4(short, TimeValue, TimeValue*, Fixed) );
		break;

	case kMediaSetSoundOutputComponentSelect:
		error=TVMediaSetSoundOutputComponent( 
		  CMP1(Component) );
		break;
	case kMediaGSetVolumeSelect: 
		error=TVMediaSetVolume( 
		  CMP1(short) );
		break;
	case kMediaSetSoundBalanceSelect: 
		error=TVMediaSetSoundBalance( 
		  CMP1(short) );
		break;

	case kMediaSetGWorldSelect: 
		error=TVMediaSetGWorld( 
		  CMP2(CGrafPtr, GDHandle) );
		break;
	case kMediaSetDimensionsSelect: 
		error=TVMediaSetDimensions( 
		  CMP2(Fixed, Fixed) );
		break;
	case kMediaSetMatrixSelect: 
		error=TVMediaSetMatrix( 
		  CMP1(MatrixRecord*) );
		break;
	case kMediaSetClipSelect: 
		error=TVMediaSetClip( 
		  CMP1(RgnHandle) );
		break;
	case kMediaSetGraphicsModeSelect:
		error=TVMediaSetGraphicsMode( 
		  CMP2(long, RGBColor const*) );
		break;

	case kMediaSetMovieTimeScaleSelect: 
		error=TVMediaSetMovieTimeScale( 
		  CMP1(TimeScale) );
		break;
	case kMediaSetMediaTimeScaleSelect:
		error=TVMediaSetMediaTimeScale( 
		  CMP1(TimeScale) );
		break;
	case kMediaTrackEditedSelect: 
		error=TVMediaTrackEdited( 
		  CMP0() );
		break;
	case kMediaGSetActiveSegmentSelect:
		error=TVMediaGSetActiveSegment( 
		  CMP2(TimeValue, TimeValue) );
		break;
	case kMediaSetRateSelect: 
		error=TVMediaSetRate( 
		  CMP1(Fixed) );
		break;

	case kMediaSetHintsSelect:
		error=TVMediaSetHints( 
		  CMP1(long) );
		break;
	case kMediaInvalidateRegionSelect:
		error=TVMediaInvalidateRegion( 
		  CMP1(RgnHandle) );
		break;

	case kMediaHasCharacteristicSelect: 
		error=TVMediaHasCharacteristic( 
		  CMP2(OSType, Boolean*) );
		break;
	case kMediaGetNameSelect:
		error=TVMediaGetName( 
		  CMP3(unsigned char*, long, long*)  );
		break;
	case kMediaGetClockSelect:
		error=TVMediaGetClock( 
		  CMP1(ComponentInstance*) );
		break;
	case kMediaGetSoundOutputComponentSelect:
		error=TVMediaGetSoundOutputComponent( 
		  CMP1(Component*) );
		break;

	default:
		error=DelegateComponentCall(paramsPtr, 
		  REINTERPRET_CAST(TVMediaHandlerRecord*, thiss)->delegateComponent);
	}

	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaVersion(
  TVMediaHandlerRecord* thiss)
{
#pragma unused(thiss)
	return kTVMediaHandlerVersion;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaCanDo(
  TVMediaHandlerRecord* thiss, short functionNumber)
{
	ASSERT(thiss!=NULL);
	ASSERT(thiss->delegateComponent!=NULL);
	
	switch(functionNumber){
	case kComponentOpenSelect:
	case kComponentCloseSelect:
	case kComponentTargetSelect: 
	case kComponentCanDoSelect:
	case kComponentVersionSelect: 

	case kMediaInitializeSelect: 
	case kMediaSetActiveSelect:

	case kMediaIdleSelect: 
	case kMediaPrerollSelect:
	case kMediaGetNextStepTimeSelect:

	case kMediaSetSoundOutputComponentSelect:
	case kMediaGSetVolumeSelect:
	case kMediaSetSoundBalanceSelect:

	case kMediaSetGWorldSelect:
	case kMediaSetDimensionsSelect:
	case kMediaSetMatrixSelect: 
	case kMediaSetClipSelect:
	case kMediaSetGraphicsModeSelect:

	case kMediaSetMovieTimeScaleSelect:
	case kMediaSetMediaTimeScaleSelect:
	case kMediaTrackEditedSelect:
	case kMediaGSetActiveSegmentSelect:
	case kMediaSetRateSelect:
	
	case kMediaSetHintsSelect:
	case kMediaInvalidateRegionSelect:

	case kMediaHasCharacteristicSelect:
	case kMediaGetNameSelect:
	case kMediaGetClockSelect:
	case kMediaGetSoundOutputComponentSelect:

		return TRUE;

	default:
		return ComponentFunctionImplemented(
		  thiss->delegateComponent, functionNumber);
	}
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaOpen(
  TVMediaHandlerRecord* dummy, ComponentInstance self)
{
#pragma unused(dummy)
/*
	The Open() function is our chance to 
	Create our shared globals if necessary. (They may already have been 
	  created by an earlier Open() hence we don't need to create them.)
	 Create our instance storage.
*/

	OSErr error;
	TVMediaHandlerGlobals* globalsPtr=NULL;

	ASSERT(self!=NULL);

	error=TVMediaNewSharedGlobalsIfNecessary( 
	  STATIC_CAST(Component, self), &globalsPtr);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	
	error=TVMediaNewInstance(self, globalsPtr);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	goto done;

cleanup:
	TVMediaDisposeSharedGlobalsIfNecessary( 
	  STATIC_CAST(Component, self), globalsPtr);

done:
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaClose(
  TVMediaHandlerRecord* thiss, ComponentInstance self)
{
/*
	The Close() function is our chance to 
	 Dispose of our instance storage.
	Dispose of our shared globals if possible. 
	  (That is, if every instance has been disposed of.)
*/

	long                   globalsLong;
	TVMediaHandlerGlobals* globalsPtr;

	//Note that <thiss> and <globalsPtr> can be NULL.
	ASSERT(self!=NULL);

	//Note that we cannot derefence <globalsPtr> off the <thiss> ptr because 
	// <thiss> may not be valid here if we had an error in Open().
	globalsLong=GetComponentRefcon( STATIC_CAST(Component, self) );
	globalsPtr=REINTERPRET_CAST(TVMediaHandlerGlobals*, globalsLong);
	TVMediaDisposeInstance(thiss, self);
	TVMediaDisposeSharedGlobalsIfNecessary( 
	  STATIC_CAST(Component, self), globalsPtr);

	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaInitialize(
  TVMediaHandlerRecord* thiss, GetMovieCompleteParams KONST* gmcPtr)
{
/*
	Initialize() is our chance to initialize our shared and instance data 
	structure.
	This initialization is deferred to this point because the Component 
	Manager often opens then closes components simply to acquire some 
	information from them and deferring costly initialization speeds up
	that process.
*/

	ComponentResult error=noErr;
	TVMediaHandlerGlobals* globalsPtr;
	ComponentInstance      self;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->self!=NULL);
	ASSERT(thiss->globalsAlias!=NULL);
	ASSERT(gmcPtr!=NULL);

	globalsPtr=thiss->globalsAlias;
	self      =thiss->self;

	error=TVMediaInitializeSharedGlobalsIfNecessary( 
	  STATIC_CAST(Component, self), globalsPtr);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	error=TVMediaInitializeInstance(thiss, gmcPtr);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	goto done;

cleanup:
	//Note that we cannot dereference globalsAlias and self off the <thiss> 
	// pointer because that may not be valid here if we have had an error.
	// (It may have been Disposed() by a function above.)
	TVMediaDisposeSharedGlobalsIfNecessary( 
	  STATIC_CAST(Component, self), globalsPtr);

	//We are not allowed to commit suicide here. Component Manager will take
	// care of calling us through Close() to dispose of this instance.
	// TVMediaDisposeInstance(thiss, self);

done:
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaTarget(
  TVMediaHandlerRecord* thiss, ComponentInstance parentComponent)
{
	ComponentResult error;

	ASSERT(thiss!=NULL);
	ASSERT(parentComponent!=NULL);
	ASSERT(thiss->delegateComponent!=NULL);

	//Remember who is at the top of our calling chain.
	thiss->parent=parentComponent;

	//Inform delegate component of the change as well.
	error=ComponentSetTarget(thiss->delegateComponent, parentComponent);
	FLAG_ERROR(error);

	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaSetActive(
  TVMediaHandlerRecord* thiss, Boolean fEnable)
{
	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);

	if(fEnable!=thiss->fEnabled){
		thiss->fEnabled=fEnable;
		if(!fEnable){
			if(thiss->sequenceGrabber!=NULL){
				ASSERT(thiss->sequenceGrabber!=NULL);
				ASSERT(thiss->sequenceGrabberVideoChannel!=NULL);
				error=SGStop(thiss->sequenceGrabber);
				if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
				SGDisposeChannel(thiss->sequenceGrabber, 
				  thiss->sequenceGrabberVideoChannel);
				CloseComponent(thiss->sequenceGrabber);
				thiss->sequenceGrabberVideoChannel=NULL;
				thiss->sequenceGrabber=NULL;
				thiss->fStartedPreview=FALSE;
			}
		}else if(thiss->sequenceGrabber==NULL){
			ASSERT(thiss->sequenceGrabber==NULL);
			ASSERT(thiss->sequenceGrabberVideoChannel==NULL);
			error=OpenADefaultComponent(SeqGrabComponentType, 0, 
			  &thiss->sequenceGrabber);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=SGInitialize(thiss->sequenceGrabber);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=SGNewChannel(thiss->sequenceGrabber, VideoMediaType, 
			  &thiss->sequenceGrabberVideoChannel);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=SGSetChannelUsage(thiss->sequenceGrabberVideoChannel, 
			  seqGrabPreview);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

			error=TVMediaSetGWorld(thiss, thiss->cgrafPtr, thiss->gdeviceH);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=TVMediaSetDimensions(thiss, thiss->fxWidth, thiss->fxHeight);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=TVMediaSetMatrix(thiss, &thiss->trackMovieMatrix);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
//			error=TVMediaSetClip(thiss, thiss->clipRgnH);
//			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			error=TVMediaSetGraphicsMode(thiss, 
			  thiss->graphicsMode, &thiss->opColor);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
		}

	}
	goto done;

cleanup:
		SGDisposeChannel(thiss->sequenceGrabber, 
		  thiss->sequenceGrabberVideoChannel);
		CloseComponent(thiss->sequenceGrabber);
		thiss->sequenceGrabberVideoChannel=NULL;
		thiss->sequenceGrabber=NULL;
done:
	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaIdle(
  TVMediaHandlerRecord* thiss, 
  TimeValue dummyTimeNow_mediaAxis, UINT32 inFlags, UINT32* outFlagsPtr,
  TimeRecord const* timeNowTR_movieAxisPtr)
{
#pragma unused (timeNowTR_movieAxisPtr, dummyTimeNow_mediaAxis)

	ComponentResult error=noErr;

	if(thiss->sequenceGrabber!=NULL){
		if(!thiss->fStartedPreview){
			thiss->fStartedPreview=TRUE;
			error=SGStartPreview(thiss->sequenceGrabber);
			FLAG_ERROR(error);

			error=SGPause(thiss->sequenceGrabber, seqGrabUnpause);
			FLAG_ERROR(error);
		}

		if(inFlags&mMustDraw || fUpdate){
			error=SGUpdate(thiss->sequenceGrabber, thiss->clipRgnH);
			FLAG_ERROR(error);
			fUpdate=FALSE;
		}

		error=SGIdle(thiss->sequenceGrabber);
		FLAG_ERROR(error);
	}

	if(outFlagsPtr!=NULL) *outFlagsPtr=mDidDraw;
	return noErr;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaPreroll(TVMediaHandlerRecord* thiss, 
  TimeValue realTimeNow_movieAxis, Fixed fxRate) 
{
#pragma unused (thiss, realTimeNow_movieAxis, fxRate)

	return noErr;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaGetNextStepTime(
  TVMediaHandlerRecord* thiss, short flags, 
  TimeValue timeIn_mediaAxis, TimeValue* timeOut_mediaAxisPtr, Fixed fxRate)
{
#pragma unused (flags)
	ComponentResult error=noErr;
	TimeValue       timeOut_mediaAxis;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->fEnabled);

	if(!thiss->fHasEyes){
		return badComponentSelector;
	}

	if(fxRate>=0){ 
		timeOut_mediaAxis
		  =timeIn_mediaAxis+thiss->mediaTimeScale;
	}else{
		timeOut_mediaAxis
		  =timeIn_mediaAxis-thiss->mediaTimeScale;
	}

	ASSERT(timeOut_mediaAxisPtr!=NULL);
	*timeOut_mediaAxisPtr=timeOut_mediaAxis;		
	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaGSetActiveSegment(
  TVMediaHandlerRecord* thiss, 
  TimeValue activeSegmentStartTime_movieAxis, 
  TimeValue activeSegmentDuration_movieScale)
{
#pragma unused (thiss, \
  activeSegmentStartTime_movieAxis, activeSegmentDuration_movieScale)

	return noErr;	
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaSetRate(
  TVMediaHandlerRecord* thiss, Fixed fxMovieRate)
{
#pragma unused (thiss, fxMovieRate)

	ComponentResult error=noErr;
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaTrackEdited(
  TVMediaHandlerRecord* thiss)
{
#pragma unused (thiss)
	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetMovieTimeScale(
  TVMediaHandlerRecord* thiss, TimeScale movieTimeScale)
{
	ASSERT(thiss!=NULL);
	ASSERT(movieTimeScale>0);

	thiss->movieTimeScale=movieTimeScale;
	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetMediaTimeScale(
  TVMediaHandlerRecord* thiss, TimeScale mediaTimeScale)
{
	ASSERT(thiss!=NULL);
	ASSERT(mediaTimeScale>0);

	thiss->mediaTimeScale=mediaTimeScale;
	return noErr;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaSetGWorld(
  TVMediaHandlerRecord* thiss, CGrafPtr cgrafPtr, GDHandle gdeviceH)
{
	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->sequenceGrabber!=NULL);

	thiss->cgrafPtr=cgrafPtr;
	thiss->gdeviceH=gdeviceH;

	if(thiss->sequenceGrabber!=NULL && thiss->fHasEyes){
		error=SGSetGWorld(thiss->sequenceGrabber, cgrafPtr, gdeviceH);
		FLAG_ERROR(error);
	}

	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetDimensions(
  TVMediaHandlerRecord* thiss, Fixed fxWidth, Fixed fxHeight)
{
	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->sequenceGrabberVideoChannel!=NULL);

	thiss->fxWidth =fxWidth;
	thiss->fxHeight=fxHeight;
	if(thiss->sequenceGrabberVideoChannel!=NULL && thiss->fHasEyes){
		ComponentResult error;
		Rect r;

		r.top   =0;
		r.left  =0;
		r.bottom=fxHeight>>16; 
		r.right =fxWidth>>16;
		error=SGSetChannelBounds(thiss->sequenceGrabberVideoChannel, &r);
		FLAG_ERROR(error);
	}
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetMatrix(
  TVMediaHandlerRecord* thiss, MatrixRecord KONST* matrixRecordPtr)
{
/*
	This is the usual way the movie is "resized". The "real" dimensions
	of the movie are unchanged but the matrix is scaled to reflect the
	size at which the movie is to be displayed.
*/	
	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(matrixRecordPtr!=NULL);

	thiss->trackMovieMatrix=*matrixRecordPtr;
	if(thiss->sequenceGrabberVideoChannel!=NULL && thiss->fHasEyes){
		FixedRect  fxr;
		FixedPoint fxpArray[4];
		Rect       r;

		fxr.top   =0; 
		fxr.left  =0; 
		fxr.bottom=thiss->fxHeight; 
		fxr.right =thiss->fxWidth;
		TransformFixedRect(matrixRecordPtr, &fxr, fxpArray);
		r.top   =(fxpArray[0].y+kfxHalf)>>16; 
		r.left  =(fxpArray[0].x+kfxHalf)>>16; 
		r.bottom=(fxpArray[2].y+kfxHalf)>>16;
		r.right =(fxpArray[2].x+kfxHalf)>>16;

		error=SGSetChannelBounds(thiss->sequenceGrabberVideoChannel, &r);
		FLAG_ERROR(error);
	}
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetClip(
  TVMediaHandlerRecord* thiss, RgnHandle rgnH)
{
/*
	Note that the media handler owns this clip region, NOT the 
	lower level mpeg handlers to which it is passed. 
	This fits with the QTC convention for stream handlers which is that 
	they copy the clipRgnH they are passed and so are not responsible 
	for the handle they are passed.
*/
	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);

	MacCopyRgn(rgnH, thiss->clipRgnH);
	DisposeRgn(rgnH);
	if(thiss->sequenceGrabberVideoChannel!=NULL && thiss->fHasEyes){
		error=SGSetChannelClip(thiss->sequenceGrabberVideoChannel, thiss->clipRgnH);
//if(error==-111) error=noErr;
		FLAG_ERROR(error);
	}
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetGraphicsMode(
  TVMediaHandlerRecord* thiss, long mode, RGBColor const* opColorPtr)
{
	ComponentResult error, error2=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->delegateComponent!=NULL);

	thiss->graphicsMode=mode;
	if(opColorPtr!=NULL) thiss->opColor=*opColorPtr;

	error=MediaSetGraphicsMode(thiss->delegateComponent, mode, opColorPtr);
	FLAG_ERROR(error);

	return noErr;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaSetSoundOutputComponent(
  TVMediaHandlerRecord* thiss, Component outputComponent)
{
#pragma unused (thiss, outputComponent)

	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetVolume(
  TVMediaHandlerRecord* thiss, short volume)
{
#pragma unused(thiss, volume)

	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaSetSoundBalance(
  TVMediaHandlerRecord* thiss, short balance)
{
#pragma unused (thiss, balance)

	return noErr;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaSetHints(
   TVMediaHandlerRecord* thiss,  long hints)
{
#pragma unused (thiss, hints)

	ComponentResult error=noErr;

	ASSERT(thiss!=NULL);

	if(hints&hintsInactive){
		error=TVMediaSetActive(thiss, FALSE);
		FLAG_ERROR(error);
	}else{
		error=TVMediaSetActive(thiss, TRUE);
		FLAG_ERROR(error);
	}

	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaInvalidateRegion(
  TVMediaHandlerRecord* thiss, RgnHandle rgnH)
{
#pragma unused(rgnH) 
/*
	This call is made as response to an update event. 
	We pass it down to our blitting component which informs the hardware
	if ICM is using key-color flooding to control regions of the screen for 
	YUV blitting.
	We also set a flag that the display has changed so that we redraw the 
	window on the next pass through Idle().
*/

	ComponentResult error=noErr;

	if(thiss->sequenceGrabber!=NULL && thiss->fHasEyes){
		//Not allowed to blit at nonIdle time, so we set a flag.
		fUpdate=TRUE;
	}

	return error;
}
///=============================================================================

STATIC COMPONENT ComponentResult TVMediaHasCharacteristic(
  TVMediaHandlerRecord* thiss,  OSType characteristic, Boolean* fHasItPtr)
{
#pragma disjoint(*thiss, *fHasItPtr)

	ASSERT(thiss!=NULL);
	ASSERT(thiss->delegateComponent);

	//First call the base media handler to see what it has to say.
	// (It will know about strange low-level characteristics I know
	// nothing about.)
	MediaHasCharacteristic(thiss->delegateComponent, 
	  characteristic, fHasItPtr);

	//If it doesn't know about a characteristic (and hence does not 
	// set <*fHasItPtr>  I step in.
	if(fHasItPtr!=NULL && *fHasItPtr==FALSE){
		switch(characteristic){
		case AudioMediaCharacteristic:
			*fHasItPtr=thiss->fHasEars;
			break;
		case VisualMediaCharacteristic:
			*fHasItPtr=thiss->fHasEyes;
			break;
		}
	}
	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaGetName(TVMediaHandlerRecord* thiss, 
  unsigned char* namePtr, long requestedLanguage, long* actualLanguagePtr)
{
#pragma unused (requestedLanguage)
#pragma disjoint(*thiss, *namePtr, *actualLanguagePtr)

#define kMediaTypeNameListStringsRes 1
#define kStringID 	                 2

	OSErr error=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->self!=NULL);

	if(namePtr!=NULL){
		if(actualLanguagePtr!=NULL){
			error=GetComponentIndString( STATIC_CAST(Component, thiss->self), 
			  namePtr, kMediaTypeNameListStringsRes, 1);
			if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
			StringToNum(namePtr, actualLanguagePtr);
		}	
		error=GetComponentIndString( STATIC_CAST(Component, thiss->self), 
		  namePtr, kMediaTypeNameListStringsRes, kStringID);
		if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	}
	goto done;

cleanup:
	//All cleanup done below.
done:
	return error;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaGetClock(
  TVMediaHandlerRecord* thiss, ComponentInstance* clockPtr)
{
#pragma unused (thiss, clockPtr)
	return noErr;
}
///-----------------------------------------------------------------------------

STATIC COMPONENT ComponentResult TVMediaGetSoundOutputComponent(
  TVMediaHandlerRecord* thiss, Component* outputComponentPtr)
{
	ASSERT(thiss!=NULL);

	if(outputComponentPtr!=NULL){
		*outputComponentPtr=thiss->soundOutputComponent;
	}
	return noErr;
}
///=============================================================================

OSErr TestIfHasEarsOrEyesCharacteristicsAndInformBaseMediaHandler(
  TVMediaHandlerRecord* thiss)
{
	OSErr  error=noErr;
	Handle sampleDescriptionH=NULL;
	UINT32 capabilities;

	ASSERT(thiss!=NULL);
	ASSERT(thiss->delegateComponent!=NULL);

	thiss->fHasEyes=TRUE;
	thiss->fHasEars=TRUE;

	//Tell the Base Media Handler about ourselves.
	capabilities=handlerNoScheduler|handlerWantsTime;
	if(thiss->fHasEyes){
		capabilities
		  |=handlerHasSpatial|handlerCanClip;
	}
	error=MediaSetHandlerCapabilities(thiss->delegateComponent, 
	  capabilities, 0xFFFFFFFF);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	if(thiss->fHasEyes){
		error=TVMediaSetGWorld(thiss, 
		  thiss->cgrafPtr, thiss->gdeviceH);
		FLAG_ERROR(error);
		error=TVMediaSetDimensions(thiss, 
		  thiss->fxWidth, thiss->fxHeight);
		FLAG_ERROR(error);
		error=TVMediaSetMatrix(thiss, 
		  CONST_CAST(MatrixRecord KONST*, &thiss->trackMovieMatrix) );
		FLAG_ERROR(error);
//		error=TVMediaSetClip(thiss, 
//		  thiss->clipRgnH);
//		FLAG_ERROR(error);
		error=TVMediaSetGraphicsMode(thiss,
		  thiss->graphicsMode, &thiss->opColor);
		FLAG_ERROR(error);
	}
	if(thiss->fHasEars){
	}

	goto done;

cleanup:
	//The only cleanup to be done is that below.
done:
	DisposeHandle(sampleDescriptionH);
	return error;
}
///=============================================================================
