#include "Maynard.h"
#include <Sound.h>

#include "TVMediaHandlerComponent.h"

///=============================================================================

OSErr TVMediaNewInstance(ComponentInstance self, 
  TVMediaHandlerGlobals* globalsPtr)
{
	OSErr error=noErr;
	TVMediaHandlerRecord* thiss;
	ComponentDescription    componentDescriptionOfSelf;

	ASSERT(self!=NULL);
	ASSERT(globalsPtr!=NULL);

	//Use NewPtrClear here to set all pointer members of the struct to NULL.
	thiss=STATIC_CAST(TVMediaHandlerRecord*, 
	  NewPtrClear( sizeof(TVMediaHandlerRecord) ) );
	if(thiss==NULL){
		error=MemError();
		FLAG_ERROR(error);
		goto cleanup;
	}
	SetComponentInstanceStorage(self, REINTERPRET_CAST(Handle, thiss) );

	//Evaluate the type of stream with which we are dealing 
	// (muxed, video-only or audio-only).
	error=GetComponentInfo( STATIC_CAST(Component, self), 
	  &componentDescriptionOfSelf, NULL, NULL, NULL);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	ASSERT(componentDescriptionOfSelf.componentSubType=='mhTV');

	//Handle targeting.
	thiss->self  =self;
	thiss->parent=self;
	//Find a Base Media Handler to whom to delegate.
	error=OpenADefaultComponent(
	  MediaHandlerType, BaseMediaType, &thiss->delegateComponent);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	//Set up the calling chain.
	CallComponentTarget(self, self);

	thiss->globalsAlias=globalsPtr;

	goto done;

cleanup:
	//No cleanup possible.
	// If something goes wrong, we are not allowed to call 
	// TVMediaDisposeInstance() and assume we're dead.
	//(A component may not commit suicide.)

done:
	return error;
}
///-----------------------------------------------------------------------------

OSErr TVMediaNewSharedGlobalsIfNecessary(Component self, 
  TVMediaHandlerGlobals** globalsPtrPtr)
{
	OSErr error=noErr;
	TVMediaHandlerGlobals* globalsPtr;
	long                   globalsLong;
	THz		 	     	   savedZone;
	BOOL				   fInAppHeap;

	ASSERT(self!=NULL);
	ASSERT(globalsPtrPtr!=NULL);

	savedZone=GetZone();
	globalsLong=GetComponentRefcon(self);
	globalsPtr=REINTERPRET_CAST(TVMediaHandlerGlobals*, globalsLong);
	if(globalsPtr==NULL){
		fInAppHeap=( 
		  GetComponentInstanceA5( STATIC_CAST(ComponentInstance, self) )
		  !=NULL );
		if(!fInAppHeap){
			SetZone( SystemZone() );
		}

		//Use NewPtrClear here to set all pointer members of the struct 
		// to NULL, and because the structure is shared.
		globalsPtr=STATIC_CAST(TVMediaHandlerGlobals*, 
		  NewPtrClear( sizeof(TVMediaHandlerGlobals) )  );
		if(globalsPtr==NULL){
			error=MemError();
			FLAG_ERROR(error);
			goto cleanup;
		} 
		globalsLong=REINTERPRET_CAST(long, globalsPtr);
		SetComponentRefcon(self, globalsLong);
	}
	
	goto done;

cleanup:
	TVMediaDisposeSharedGlobalsIfNecessary(self, globalsPtr);
	globalsPtr=NULL;

done:
	SetZone(savedZone);

	*globalsPtrPtr=globalsPtr;
	return error;
}
///-----------------------------------------------------------------------------

void TVMediaDisposeInstance(TVMediaHandlerRecord* thiss, ComponentInstance self)
{
	ASSERT(self!=NULL);

	if(thiss!=NULL){
		CloseComponent(thiss->delegateComponent);

		SGStop(thiss->sequenceGrabber);
		SGDisposeChannel(thiss->sequenceGrabber, thiss->sequenceGrabberVideoChannel);
		CloseComponent(thiss->sequenceGrabber);

		DisposeHandle(thiss->sampleDescriptionH);
		DisposeRgn(thiss->clipRgnH);

		DisposePtr( STATIC_CAST(Ptr, thiss) );
	}
	SetComponentInstanceStorage(self, NULL);
}
///-----------------------------------------------------------------------------

void TVMediaDisposeSharedGlobalsIfNecessary(Component self, 
  TVMediaHandlerGlobals* globalsPtr)
{
	ASSERT(self!=NULL);

	if( CountComponentInstances(self)<=1 ){
		if(globalsPtr!=NULL){
			DisposePtr( STATIC_CAST(Ptr, globalsPtr) );
		}
		SetComponentRefcon(self, NULL);
	}
}
///-----------------------------------------------------------------------------

OSErr TVMediaInitializeInstance(TVMediaHandlerRecord* thiss,
  GetMovieCompleteParams const* gmcPtr)
{
	OSErr error=noErr;

	ASSERT(thiss!=NULL);
	ASSERT(gmcPtr!=NULL);
	ASSERT(thiss->delegateComponent!=NULL);

	//Remember some useful parameters from gmcPtr.
	thiss->movie           =gmcPtr->theMovie;
	thiss->track           =gmcPtr->theTrack;
	thiss->media           =gmcPtr->theMedia;
	thiss->movieTimeScale  =gmcPtr->movieScale;
	thiss->mediaTimeScale  =gmcPtr->mediaScale;
	thiss->volume          =gmcPtr->volume;
	thiss->fxEffectiveClockRate  =gmcPtr->effectiveRate;
	thiss->fxWidth		   =gmcPtr->width;
	thiss->fxHeight        =gmcPtr->height;
	thiss->trackMovieMatrix=gmcPtr->trackMovieMatrix;
	thiss->cgrafPtr        =gmcPtr->moviePort;
	thiss->gdeviceH        =gmcPtr->movieGD;
	thiss->movieTimeBase   =gmcPtr->timeBase;

	ASSERT(thiss->movie!=NULL);
	ASSERT(thiss->track!=NULL);
	ASSERT(thiss->media!=NULL);
	ASSERT(thiss->movieTimeScale>0);
	ASSERT(thiss->mediaTimeScale>0);
	ASSERT(thiss->movieTimeBase!=NULL);

	error=MediaGetSoundBalance(thiss->delegateComponent, &thiss->balance);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	thiss->clipRgnH=NewRgn();
	if(thiss->clipRgnH==NULL){
		error=memFullErr; FLAG_ERROR(error); goto cleanup;
	}

	error=OpenADefaultComponent(SeqGrabComponentType, 0, 
	  &thiss->sequenceGrabber);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	error=SGInitialize(thiss->sequenceGrabber);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	error=SGNewChannel(thiss->sequenceGrabber, VideoMediaType, 
	  &thiss->sequenceGrabberVideoChannel);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}
	error=SGSetChannelUsage(thiss->sequenceGrabberVideoChannel, 
	  seqGrabPreview);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}


	//Note that this handle will be resized as appropriate when the 
	// sampleDescription (which is variable-sized) is filled in.
	thiss->sampleDescriptionH=NewHandle(0);
	if(thiss->sampleDescriptionH==NULL){
		error=MemError(); FLAG_ERROR(error); goto cleanup;
	}

	error=TestIfHasEarsOrEyesCharacteristicsAndInformBaseMediaHandler(thiss);
	if(error!=noErr) {FLAG_ERROR(error); goto cleanup;}

	goto done;

cleanup:
	//No cleanup possible.
	// If something goes wrong, we are not allowed to call 
	// TVMediaDisposeInstance() and assume we're dead.
	//(A component may not commit suicide.)
done:
	return noErr; //error;
}
///-----------------------------------------------------------------------------

OSErr TVMediaInitializeSharedGlobalsIfNecessary(Component self,
   TVMediaHandlerGlobals* globalsAlias)
{
	OSErr error=noErr;
	THz	 savedZone;

	ASSERT(self!=NULL);
	ASSERT(globalsAlias!=NULL);
	
	savedZone=GetZone();
	goto done;

cleanup:
	TVMediaDisposeSharedGlobalsIfNecessary(self, globalsAlias);
done:
	SetZone(savedZone);	
	return error;
}
///=============================================================================
